/**
  ******************************************************************************
  * @file    usb_regs.h
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    16-January-2014
  * @brief   Interface prototype functions to USB cell registers
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __USB_REGS_H
#define __USB_REGS_H

/* Exported types ------------------------------------------------------------*/
typedef enum _EP_DBUF_DIR
{
  /* double buffered endpoint direction */
  EP_DBUF_OUT,
  EP_DBUF_IN,
  EP_DBUF_ERR,
}EP_DBUF_DIR;

/* endpoint buffer number */
enum EP_BUF_NUM
{
  EP_NOBUF,
  EP_BUF0,
  EP_BUF1
};

/* Exported defines ----------------------------------------------------------*/
#define RegBase	 (0x40005C00L)	/* USB_IP Peripheral Registers base address */
#define PMAAddr	 (0x40006000L)	/* USB_IP Packet Memory Area base address   */

/******************************************************************************/
/*			   General registers				      */
/******************************************************************************/

/* Control register */
#define CNTR	((__IO unsigned *)(RegBase + 0x40))
/* Interrupt status register */
#define ISTR	((__IO unsigned *)(RegBase + 0x44))
/* Frame number register */
#define FNR	((__IO unsigned *)(RegBase + 0x48))
/* Device address register */
#define DADDR	((__IO unsigned *)(RegBase + 0x4C))
/* Buffer Table address register */
#define BTABLE	((__IO unsigned *)(RegBase + 0x50))

/* Battery Charging detector register*/
#define BCDR	(( __IO unsigned *)(RegBase + 0x58))
  /* LPM Control and Status register */
#define LPMCSR	  (( __IO unsigned *)(RegBase + 0x54))

/******************************************************************************/
/*			   Endpoint registers				      */
/******************************************************************************/
#define EP0REG	((__IO unsigned *)(RegBase)) /* endpoint 0 register address */

/* Endpoint Addresses (w/direction) */
#define EP0_OUT	    ((uint8_t)0x00)
#define EP0_IN	    ((uint8_t)0x80)
#define EP1_OUT	    ((uint8_t)0x01)
#define EP1_IN	    ((uint8_t)0x81)
#define EP2_OUT	    ((uint8_t)0x02)
#define EP2_IN	    ((uint8_t)0x82)
#define EP3_OUT	    ((uint8_t)0x03)
#define EP3_IN	    ((uint8_t)0x83)
#define EP4_OUT	    ((uint8_t)0x04)
#define EP4_IN	    ((uint8_t)0x84)
#define EP5_OUT	    ((uint8_t)0x05)
#define EP5_IN	    ((uint8_t)0x85)
#define EP6_OUT	    ((uint8_t)0x06)
#define EP6_IN	    ((uint8_t)0x86)
#define EP7_OUT	    ((uint8_t)0x07)
#define EP7_IN	    ((uint8_t)0x87)

/* endpoints enumeration */
#define ENDP0	    ((uint8_t)0)
#define ENDP1	    ((uint8_t)1)
#define ENDP2	    ((uint8_t)2)
#define ENDP3	    ((uint8_t)3)
#define ENDP4	    ((uint8_t)4)
#define ENDP5	    ((uint8_t)5)
#define ENDP6	    ((uint8_t)6)
#define ENDP7	    ((uint8_t)7)

/******************************************************************************/
/*			 ISTR interrupt events				      */
/******************************************************************************/
#define ISTR_CTR    (0x8000) /* Correct TRansfer (clear-only bit) */
#define ISTR_DOVR   (0x4000) /* DMA OVeR/underrun (clear-only bit) */
#define ISTR_ERR    (0x2000) /* ERRor (clear-only bit) */
#define ISTR_WKUP   (0x1000) /* WaKe UP (clear-only bit) */
#define ISTR_SUSP   (0x0800) /* SUSPend (clear-only bit) */
#define ISTR_RESET  (0x0400) /* RESET (clear-only bit) */
#define ISTR_SOF    (0x0200) /* Start Of Frame (clear-only bit) */
#define ISTR_ESOF   (0x0100) /* Expected Start Of Frame (clear-only bit) */
#define ISTR_L1REQ  (0x0080)  /* LPM L1 state request  */
#define ISTR_DIR    (0x0010)  /* DIRection of transaction (read-only bit)  */
#define ISTR_EP_ID  (0x000F)  /* EndPoint IDentifier (read-only bit)  */

#define CLR_CTR	   (~ISTR_CTR)	 /* clear Correct TRansfer bit */
#define CLR_DOVR   (~ISTR_DOVR)	 /* clear DMA OVeR/underrun bit*/
#define CLR_ERR	   (~ISTR_ERR)	 /* clear ERRor bit */
#define CLR_WKUP   (~ISTR_WKUP)	 /* clear WaKe UP bit	  */
#define CLR_SUSP   (~ISTR_SUSP)	 /* clear SUSPend bit	  */
#define CLR_RESET  (~ISTR_RESET) /* clear RESET bit	 */
#define CLR_SOF	   (~ISTR_SOF)	 /* clear Start Of Frame bit   */
#define CLR_ESOF   (~ISTR_ESOF)	 /* clear Expected Start Of Frame bit */
#define CLR_L1REQ  (~ISTR_L1REQ)  /* clear LPM L1  bit */

/******************************************************************************/
/*	       CNTR control register bits definitions			      */
/******************************************************************************/
#define CNTR_CTRM   (0x8000) /* Correct TRansfer Mask */
#define CNTR_DOVRM  (0x4000) /* DMA OVeR/underrun Mask */
#define CNTR_ERRM   (0x2000) /* ERRor Mask */
#define CNTR_WKUPM  (0x1000) /* WaKe UP Mask */
#define CNTR_SUSPM  (0x0800) /* SUSPend Mask */
#define CNTR_RESETM (0x0400) /* RESET Mask   */
#define CNTR_SOFM   (0x0200) /* Start Of Frame Mask */
#define CNTR_ESOFM  (0x0100) /* Expected Start Of Frame Mask */
#define CNTR_L1REQM (0x0080)	/* LPM L1 state request interrupt mask */
#define CNTR_L1RESUME (0x0020) /* LPM L1 Resume request */
#define CNTR_RESUME (0x0010) /* RESUME request */
#define CNTR_FSUSP  (0x0008) /* Force SUSPend */
#define CNTR_LPMODE (0x0004) /* Low-power MODE */
#define CNTR_PDWN   (0x0002) /* Power DoWN */
#define CNTR_FRES   (0x0001) /* Force USB RESet */

/******************************************************************************/
/*	       BCDR control register bits definitions			      */
/******************************************************************************/
#define	 BCDR_DPPU	((uint16_t)0x8000) /* DP Pull-up Enable	      */
#define	 BCDR_PS2DET	((uint16_t)0x0080) /* PS2 port or proprietary charger detected */
#define	 BCDR_SDET	((uint16_t)0x0040) /* Secondary detection (SD) status	*/
#define	 BCDR_PDET	((uint16_t)0x0020) /* Primary detection (PD) status    */
#define	 BCDR_DCDET	((uint16_t)0x0010) /* Data contact detection (DCD) status  */
#define	 BCDR_SDEN	((uint16_t)0x0008) /* Secondary detection (SD) mode enable */
#define	 BCDR_PDEN	((uint16_t)0x0004) /* Primary detection (PD) mode enable */
#define	 BCDR_DCDEN	((uint16_t)0x0002) /* Data contact detection (DCD) mode enable*/
#define	 BCDR_BCDEN	((uint16_t)0x0001) /* Battery charging detector (BCD) enable  */

/******************************************************************************/
/*	       Bit definition for LPM register			       */
/******************************************************************************/
#define	 LPMCSR_BESL	((uint16_t)0x00F0) /* BESL value received with last ACKed LPM Token  */
#define	 LPMCSR_REMWAKE ((uint16_t)0x0008) /* bRemoteWake value received with last ACKed LPM Token */
#define	 LPMCSR_LPMACK	((uint16_t)0x0002) /* LPM Token acknowledge enable*/
#define	 LPMCSR_LMPEN	((uint16_t)0x0001) /* LPM support enable  */

/******************************************************************************/
/*		  FNR Frame Number Register bit definitions		      */
/******************************************************************************/
#define FNR_RXDP (0x8000) /* status of D+ data line */
#define FNR_RXDM (0x4000) /* status of D- data line */
#define FNR_LCK	 (0x2000) /* LoCKed */
#define FNR_LSOF (0x1800) /* Lost SOF */
#define FNR_FN	(0x07FF) /* Frame Number */
/******************************************************************************/
/*		 DADDR Device ADDRess bit definitions			      */
/******************************************************************************/
#define DADDR_EF (0x80)
#define DADDR_ADD (0x7F)
/******************************************************************************/
/*			      Endpoint register				      */
/******************************************************************************/
/* bit positions */
#define EP_CTR_RX      (0x8000) /* EndPoint Correct TRansfer RX */
#define EP_DTOG_RX     (0x4000) /* EndPoint Data TOGGLE RX */
#define EPRX_STAT      (0x3000) /* EndPoint RX STATus bit field */
#define EP_SETUP       (0x0800) /* EndPoint SETUP */
#define EP_T_FIELD     (0x0600) /* EndPoint TYPE */
#define EP_KIND	       (0x0100) /* EndPoint KIND */
#define EP_CTR_TX      (0x0080) /* EndPoint Correct TRansfer TX */
#define EP_DTOG_TX     (0x0040) /* EndPoint Data TOGGLE TX */
#define EPTX_STAT      (0x0030) /* EndPoint TX STATus bit field */
#define EPADDR_FIELD   (0x000F) /* EndPoint ADDRess FIELD */

/* EndPoint REGister MASK (no toggle fields) */
#define EPREG_MASK     (EP_CTR_RX|EP_SETUP|EP_T_FIELD|EP_KIND|EP_CTR_TX|EPADDR_FIELD)

/* EP_TYPE[1:0] EndPoint TYPE */
#define EP_TYPE_MASK   (0x0600) /* EndPoint TYPE Mask */
#define EP_BULK	       (0x0000) /* EndPoint BULK */
#define EP_CONTROL     (0x0200) /* EndPoint CONTROL */
#define EP_ISOCHRONOUS (0x0400) /* EndPoint ISOCHRONOUS */
#define EP_INTERRUPT   (0x0600) /* EndPoint INTERRUPT */
#define EP_T_MASK      (~EP_T_FIELD & EPREG_MASK)

/* EP_KIND EndPoint KIND */
#define EPKIND_MASK    (~EP_KIND & EPREG_MASK)

/* STAT_TX[1:0] STATus for TX transfer */
#define EP_TX_DIS      (0x0000) /* EndPoint TX DISabled */
#define EP_TX_STALL    (0x0010) /* EndPoint TX STALLed */
#define EP_TX_NAK      (0x0020) /* EndPoint TX NAKed */
#define EP_TX_VALID    (0x0030) /* EndPoint TX VALID */
#define EPTX_DTOG1     (0x0010) /* EndPoint TX Data TOGgle bit1 */
#define EPTX_DTOG2     (0x0020) /* EndPoint TX Data TOGgle bit2 */
#define EPTX_DTOGMASK  (EPTX_STAT|EPREG_MASK)

/* STAT_RX[1:0] STATus for RX transfer */
#define EP_RX_DIS      (0x0000) /* EndPoint RX DISabled */
#define EP_RX_STALL    (0x1000) /* EndPoint RX STALLed */
#define EP_RX_NAK      (0x2000) /* EndPoint RX NAKed */
#define EP_RX_VALID    (0x3000) /* EndPoint RX VALID */
#define EPRX_DTOG1     (0x1000) /* EndPoint RX Data TOGgle bit1 */
#define EPRX_DTOG2     (0x2000) /* EndPoint RX Data TOGgle bit1 */
#define EPRX_DTOGMASK  (EPRX_STAT|EPREG_MASK)

#endif /* __USB_REGS_H */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
