/**
  ******************************************************************************
  * @file    usbd_pwr.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    17-January-2014
  * @brief   This file provides functions for power management
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "usbd_pwr.h"

/** @addtogroup STM32_USB_DEVICE_LIBRARY
* @{
*/

/** @defgroup USBD_PWR
* @brief    This file provides power management functions
* @{
*/

/** @defgroup USBD_PWR_Private_TypesDefinitions
* @{
*/
/**
* @}
*/


/** @defgroup USBD_PWR_Private_Defines
* @{
*/
/**
* @}
*/


/** @defgroup USBD_PWR_Private_Macros
* @{
*/
/**
* @}
*/


/** @defgroup USBD_PWR_Private_Variables
* @{
*/

struct
{
  __IO RESUME_STATE eState;
  __IO uint8_t bESOFcnt;
}
ResumeS;


/**
* @}
*/

/** @defgroup USBD_PWR_Private_Constants
* @{
*/

/**
* @}
*/



/** @defgroup USBD_PWR_Private_FunctionPrototypes
* @{
*/
/**
* @}
*/


/** @defgroup USBD_PWR_Private_Functions
* @{
*/


/**
* @brief  Suspend
*	  Sets suspend mode operating conditions
* @param  None
* @retval USB_SUCCESS
*/
void Suspend(void)
{
  uint16_t wCNTR;

  /*Store CNTR value */
  wCNTR = _GetCNTR();
  /* Set FSUSP bit in USB_CNTR register*/
  wCNTR |= CNTR_FSUSP;
  _SetCNTR(wCNTR);

  /* force low-power mode in the macrocell */
  wCNTR = _GetCNTR();
  wCNTR |= CNTR_LPMODE;
  _SetCNTR(wCNTR);

#ifdef USB_DEVICE_LOW_PWR_MGMT_SUPPORT

  /* enter system in STOP mode, only when wakeup flag in not set */
  if((_GetISTR()&ISTR_WKUP)==0)
  {
    /*Enter STOP mode with SLEEPONEXIT*/
    PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_SLEEPONEXIT);
  }
  else
  {
    /* Clear Wakeup flag */
    _SetISTR(CLR_WKUP);
    /* clear FSUSP to abort entry in suspend mode  */
    wCNTR = _GetCNTR();
    wCNTR&=~CNTR_FSUSP;
    _SetCNTR(wCNTR);
  }
#endif
}

/**
* @brief  Resume_Init
*	  Handles wake-up restoring normal operations
* @param  None
* @retval USB_SUCCESS
*/
void Resume_Init(void)
{
  uint16_t wCNTR;

  /* ------------------ ONLY WITH BUS-POWERED DEVICES ---------------------- */
  /* restart the clocks */
  /* ...  */

  /* CNTR_LPMODE = 0 */
  wCNTR = _GetCNTR();
  wCNTR &= (~CNTR_LPMODE);
  _SetCNTR(wCNTR);
#ifdef USB_DEVICE_LOW_PWR_MGMT_SUPPORT
  /* restore full power */
  /* ... on connected devices */
  Leave_LowPowerMode();
#endif
  /* reset FSUSP bit */
  _SetCNTR(IMR_MSK);

}


/**
* @brief  Resume
*	  This is the state machine handling resume operations and
*	  timing sequence. The control is based on the Resume structure
*	  variables and on the ESOF interrupt calling this subroutine
*	  without changing machine state.
* @param  a state machine value (RESUME_STATE)
*	  RESUME_ESOF doesn't change ResumeS.eState allowing
*	  decrementing of the ESOF counter in different states.
* @retval Staus
*/
void Resume(RESUME_STATE eResumeSetVal)
{
 uint16_t wCNTR;
 __IO uint32_t remotewakeupon=0;

  if (eResumeSetVal != RESUME_ESOF)
    ResumeS.eState = eResumeSetVal;
  switch (ResumeS.eState)
  {
    case RESUME_EXTERNAL:

if (remotewakeupon ==0)
      {
	Resume_Init();
	ResumeS.eState = RESUME_OFF;
      }
      else /* RESUME detected during the RemoteWAkeup signalling => keep RemoteWakeup handling*/
      {
	ResumeS.eState = RESUME_ON;
      }
      break;
    case RESUME_INTERNAL:
      Resume_Init();
      ResumeS.eState = RESUME_START;
      remotewakeupon = 1;
      break;
    case RESUME_LATER:
      ResumeS.bESOFcnt = 2;
      ResumeS.eState = RESUME_WAIT;
      break;
    case RESUME_WAIT:
      ResumeS.bESOFcnt--;
      if (ResumeS.bESOFcnt == 0)
	ResumeS.eState = RESUME_START;
      break;
    case RESUME_START:
      wCNTR = _GetCNTR();
      wCNTR |= CNTR_RESUME;
      _SetCNTR(wCNTR);
      ResumeS.eState = RESUME_ON;
      ResumeS.bESOFcnt = 10;
      break;
    case RESUME_ON:
      ResumeS.bESOFcnt--;
      if (ResumeS.bESOFcnt == 0)
      {
	wCNTR = _GetCNTR();
	wCNTR &= (~CNTR_RESUME);
	_SetCNTR(wCNTR);
	ResumeS.eState = RESUME_OFF;
	remotewakeupon = 0;
      }
      break;
    case RESUME_OFF:
    case RESUME_ESOF:
    default:
      ResumeS.eState = RESUME_OFF;
      break;
  }
}

/**
* @brief  Leave_LowPowerMode
*	  Restores system clocks and power while exiting suspend mode
* @param  None
* @retval None
*/
void Leave_LowPowerMode(void)
{

  /* Enable HSI48 oscillator */
  RCC_HSI48Cmd(ENABLE);

  /* Wait till HSI48RDYF is set */
  while(RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) == RESET)
  {
  }
  /* Select HSI48 as system clock source */
  RCC_SYSCLKConfig (RCC_SYSCLKSource_HSI48);

#ifndef USB_CLOCK_SOURCE_CRS

  /* After wake-up from STOP mode restore USB clock source (USB clock = PLL clock
  from HSE source )*/
  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET)
  {}

  /* Select HSE as PLL source */
  RCC_PLLConfig (RCC_PLLSource_HSE, RCC_PLLMul_8);
  /* Enable PLL */
  RCC_PLLCmd(ENABLE);

  /* Wait till PLL is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
  {}
#endif

  /*Low Power Sleep on Exit Disabled*/
  NVIC_SystemLPConfig(NVIC_LP_SLEEPONEXIT, DISABLE);
}

/**
* @}
*/

/**
* @}
*/

/**
* @}
*/

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
